<?php
/**
 * Created by PhpStorm.
 * User: Alexander
 * Date: 31.01.2017
 * Time: 10:21
 */

namespace UBRIR;

use MongoDB\Driver\Server;

include_once 'UBRIR_TypeEnums.php';
include_once 'UBRIR_ResultViewer.php';
include_once 'UBRIR_DataContainer.php';
include_once 'UBRIR_CMS_Payment.php';
include_once 'UBRIR_ServerConnector.php';

class OpenCart extends CMS_Payment
{
    protected $ModuleRootDir;

    function __construct()
    {
        $this->workMode = 'NORMAL';
        $this->ModuleRootDir = DIR_SYSTEM . 'vendor' . DIRECTORY_SEPARATOR . 'ubrr' . DIRECTORY_SEPARATOR;

        $this->logDir = $this->ModuleRootDir . 'logs/';
        $this->logFile = $this->logDir . 'ubrr.log';

        $this->useLogs = true; //По умолчанию для тестового режиме
    }

    public function CleanDebugLog()
    {
        if (file_exists($this->logFile)) {
            return unlink($this->logFile);
        }
        return false;
    }

    public function GetDebugLogText()
    {
        if (file_exists($this->logFile)) {
            //TODO: Тут стоит читать с конца + явно только часть, длинный лог пихать в память смысла нет
            return file_get_contents($this->logFile);
        }
    }

    /**
     * Проверяем что все необходимые поля для подготовки оплаты переданы и можно формировать XML к отправке
     * [Метод пишет в лог, начинает и заканчивает]
     */
    public function PrepareToPay()
    {
        $this->Log('[START]');

        $checkSecurityResponse = $this->CheckSecurityConfiguration();
        if (!$checkSecurityResponse->isSuccess()) {
            $this->Log("[END]\r\n\r\n");
            return $checkSecurityResponse;
        }

        //Проверка на заполненность всех необходимых полей
        $Amount = $this->getAmount();
        $Merchant = $this->getShopId();

        $ApproveURL = $this->getApproveURL();
        $CancelURL = $this->getCancelURL();
        $DeclineURL = $this->getDeclineURL();

        $serverResponse = ServerConnector::PrepareToPay($this);

        $this->Log('[END]');
        return $serverResponse;
    }

    //Заполнение необходимых параметров и затем попытка передачи
    public function SetupAndPrepareToPay($orderId, $orderAmount, $approveCallbackURL, $cancelCallbackURL = '', $declineCallbackURL = '')
    {
        $this->orderId = $orderId;
        $this->orderAmount = $orderAmount;

        if (empty($cancelCallbackURL))
            $cancelCallbackURL = $approveCallbackURL;

        if (empty($declineCallbackURL))
            $declineCallbackURL = $approveCallbackURL;

        $this->approveUrl = $approveCallbackURL;
        $this->cancelUrl = $cancelCallbackURL;
        $this->declineUrl = $declineCallbackURL;

        return $this->PrepareToPay();
    }

    public function SetupAndCheckOrderStatus($TWPG_OrderId, $SessionId)
    {
        $this->ubrrOrderId = $TWPG_OrderId;
        $this->ubrrSessionId = $SessionId;

        return $this->CheckOrderStatus();
    }

    public function CheckOrderStatus()
    {
        $this->Log('[START]');

        $checkSecurityResponse = $this->CheckSecurityConfiguration();
        if (!$checkSecurityResponse->isSuccess()) {
            $this->Log("[END]\r\n\r\n");
            return $checkSecurityResponse;
        }

        //Проверка на заполненность всех необходимых полей
        $TWPG_OrderId = $this->getTWPG_OrderId();
        $TWPG_SessionId = $this->getTWPG_SessionId();

        $serverResponse = ServerConnector::CheckOrderStatus($this);

        $this->Log('[END]');
        return $serverResponse;
    }


    public static function SaveTWPGOrder($db, $orderID, $TWPG_OrderId, $SessionId)
    {
        $sql = "INSERT INTO `" . DB_PREFIX . "twpg_orders` (`shoporderid`, `OrderID`, `SessionID`) 
                VALUES ('" . $orderID . "', '" . $TWPG_OrderId . "', '" . $SessionId . "') 
                ON DUPLICATE KEY UPDATE `shoporderid`='" . $orderID . "', `OrderID`='" . $TWPG_OrderId . "', `SessionID`='" . $SessionId . "'";

        $dbResult = $db->query($sql);
        return $dbResult;
    }

    public static function GetTWPGDataByUBRIROrderID($db, $TWPG_OrderId)
    {
        $sql = 'SELECT * FROM `' . DB_PREFIX . 'twpg_orders` WHERE `OrderID`=' . $TWPG_OrderId;
        $sql_response = $db->query($sql);

        if ($sql_response->num_rows >= 1) {
            $sql_response = $sql_response->row;
            $orderID = $sql_response['shoporderid'];
            $TWPG_OrderId = $sql_response['OrderID'];
            $SessionId = $sql_response['SessionID'];

            return array(
                'order_id' => $orderID,
                'twpg_order_id' => $TWPG_OrderId,
                'twpg_session_id' => $SessionId
            );
        }
        return false;
    }

    public static function GetTWPGDataByOrderId($db, $shopOrderId)
    {
        $sql_req = 'SELECT * FROM `' . DB_PREFIX . 'twpg_orders` WHERE `shoporderid`=' . $shopOrderId;
        $sql_response = $db->query($sql_req);
        if ($sql_response->num_rows >= 1) {
            $sql_response = $sql_response->row;
            $TWPG_OrderId = $sql_response['OrderID'];
            $SessionId = $sql_response['SessionID'];
            return array(
                'twpg_order_id' => $TWPG_OrderId,
                'twpg_session_id' => $SessionId
            );
        }
        return false;
    }
}